@@ -13,9 +13,10 @@ void atexit_handler() {
 int real_main(int argc, char *argv[]) {
   ruin_window_t *w = NULL;
   FILE *log = NULL;
+  int i = 1;
 
-  if (argc < 3) {
-    printf("%s: Usage: %s [filename] xul|xhtml\n", argv[0], argv[0]);
+  if (argc < 2) {
+    printf("%s: Usage: %s [filename] [filename] ...", argv[0], argv[0]);
     return 1;
   }
 
@@ -35,18 +36,14 @@ int real_main(int argc, char *argv[]) {
   }
 
   w = ruin_window_new(stdscr, log);
-  if (strcmp(argv[2], "xul") == 0)
-    ruin_draw_file(w, argv[1], RUIN_XML_DIALECT_XUL);
-  else if (strcmp(argv[2], "xhtml") == 0)
-    ruin_draw_file(w, argv[1], RUIN_XML_DIALECT_XHTML);
-  else {
-    printf("Invalid XML dialect specification.\n");
-    return 1;
-  }
+  for (; i < argc; i++) {
+    int ch;
 
-  { int ch;
-    while (ch = getch())
-    ruin_input_send(w, ch);
+    ruin_draw_file(w, argv[i]);
+    while (ch = getch()) {
+      if (ch == 'n') break;
+      else ruin_input_send(w, ch);
+    }
   } 
  
   pause();
@@ -58,7 +55,6 @@ int real_main(int argc, char *argv[]) {
 }
 
 int main(int argc, char *argv[]) {
-  /* scm_boot_guile(argc, argv, (void (*) ()) real_main, NULL); */
   scm_init_guile();
   real_main(argc, argv);
   return 0;
