@@ -31,7 +31,7 @@ SCM ruin_scm_api_css_get_style(SCM node, SCM property) {
   ruin_element_t *elt = ruin_window_lookup_scm(node);
   if (elt != NULL) {
     char *style = NULL;
-    style = ruin_css_lookup(elt, scm_to_locale_string(property));
+    style = ruin_css_lookup(elt, scm_to_locale_string(property), NULL);
     return scm_mem2string(style, strlen(style));
   }
   return SCM_EOL;
