@@ -1,5 +1,5 @@
 /* node-fns.c: Node-management functions for the libRUIN Scheme API
- * Copyright (C) 2006 Julian Graham
+ * Copyright (C) 2007 Julian Graham
  *
  * libRUIN is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -39,22 +39,24 @@ SCM ruin_scm_api_node_pseudo_element_handler(SCM pe, SCM doc, SCM node) {
   int i = 0, len = 0;
 
   ruin_element_t *t = ruin_window_lookup_scm(node);
+  ruin_util_list *current_ptr = t->parent_window->current_pseudo_elements;
   
   if (t == NULL)
     return SCM_BOOL_F;
 
-  len = ruin_util_list_length(t->current_pseudo_elements);
+  len = ruin_util_list_length(t->parent_window->current_pseudo_elements);
   for (i = 0; i < len; i++) {
-    char *e = ruin_util_list_peek_after(t->current_pseudo_elements, i);
+    char *e = current_ptr->data;
     if (strcmp(e, pelt_name) == 0)
       return SCM_BOOL_T;
+    current_ptr = current_ptr->next;
   }
   
   return SCM_BOOL_F;   
 }
 
 static int is_link(ruin_element_t *t) {
-  if (t->dialect == RUIN_XML_DIALECT_XHTML) {
+  if (t->dialect == RUIN_LAYOUT_XML_DIALECT_XHTML) {
     if (strcmp(ruin_scheme_sdom_get_dom_property(t->element, "sdom:node-name"),
 	       "a") == 0 &&
 	ruin_scheme_sdom_get_attribute(t->element, "href") != NULL)
@@ -64,7 +66,7 @@ static int is_link(ruin_element_t *t) {
 }
 
 static int has_visited_target(ruin_element_t *t) {
-  if (t->dialect == RUIN_XML_DIALECT_XHTML) {
+  if (t->dialect == RUIN_LAYOUT_XML_DIALECT_XHTML) {
     char *href = ruin_scheme_sdom_get_attribute(t->element, "href");
     if (strcmp(href, "") == 0)
       return TRUE;
