@@ -1,5 +1,5 @@
 /* css.h: Prototypes and declarations for css.c
- * Copyright (C) 2005 Julian Graham
+ * Copyright (C) 2007 Julian Graham
  *
  * libRUIN is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,10 +22,10 @@
 #include "layout.h"
 #include "util.h"
 
-char *ruin_css_lookup(ruin_element_t *, char *);
+char *ruin_css_lookup(ruin_element_t *, char *, ruin_util_list *);
 enum ruin_layout_foreground_color ruin_css_match_foreground_color(char *);
 enum ruin_layout_background_color ruin_css_match_background_color
 (char *, ruin_util_list *);
 void ruin_css_clear_style_cache(ruin_element_t *);
 
-#endif
+#endif /* RUIN_CSS_H */
