@@ -1,5 +1,5 @@
 /* dialect.h: Prototypes and declarations for dialect.c
- * Copyright (C) 2005 Julian Graham
+ * Copyright (C) 2007 Julian Graham
  *
  * libRUIN is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,13 +22,18 @@
 #include <libguile.h>
 
 #include "layout.h"
+#include "util.h"
 
 SCM ruin_dialect_parse_in_document_style(SCM, SCM);
-ruin_element_t *ruin_dialect_add_table_nodes(ruin_element_t *);
+
+void ruin_dialect_add_table_children(ruin_element_t *);
+void ruin_dialect_add_table_parents(ruin_element_t *);
+void ruin_dialect_add_table_columns(ruin_element_t *);
+
 void ruin_dialect_update_tab_position(ruin_element_t *, int);
 char *ruin_dialect_get_node_name(SCM node);
 ruin_element_t *ruin_dialect_generate_text_node(SCM node, 
 						ruin_element_t *,
 						ruin_element_t *);
 
-#endif
+#endif /* RUIN_DIALECT_H */
