@@ -1,5 +1,5 @@
 /* focus.c: Default focus event handlers for libRUIN
- * Copyright (C) 2006 Julian Graham
+ * Copyright (C) 2007 Julian Graham
  *
  * libRUIN is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -39,7 +39,6 @@ SCM ruin_scm_default_handler_tab(SCM event) {
 }
 
 SCM ruin_scm_default_handler_focus_toggle(SCM event) {
-  ruin_util_list *inh = ruin_util_list_new();
   SCM target = scm_call_2(scm_c_eval_string("sdom:get-event-property"),
 			  event, scm_makfrom0str("sdom:target"));
 
@@ -56,9 +55,8 @@ SCM ruin_scm_default_handler_focus_toggle(SCM event) {
   ruin_scm_api_css_set_style_x(target, bgcolorp, bgcolor);
 
   ruin_render_render_tree
-    (ruin_window_lookup_scm(target)->parent_window->top, inh);
+    (ruin_window_lookup_scm(target)->parent_window->top, NULL);
 
-  ruin_util_list_free(inh);
   refresh();
   return SCM_BOOL_T;
 }
