@@ -20,6 +20,8 @@
 #include <libguile.h>
 
 #include "input.h"
+#include "load.h"
+#include "scheme.h"
 #include "window.h"
 
 enum ruin_key_identifier _ruin_input_char_to_key_identifier(int ch) {
@@ -372,8 +374,11 @@ void ruin_input_send(ruin_window_t *w, int ch) {
 
   SCM key_loc = scm_c_eval_string("sdom:dom-key-location-standard");
 
+  if (w->focused == NULL)
+    return;
+
   ruin_util_log(w, "dispatching event sdom:event-keydown for key '%s'\n",
-		SCM_STRING_CHARS(key_str));
+		scm_to_locale_string(key_str));
   
   scm_apply_0
     (scm_c_eval_string("sdom:dispatch-event"), 
@@ -382,7 +387,7 @@ void ruin_input_send(ruin_window_t *w, int ch) {
 		SCM_BOOL_F, SCM_BOOL_F, SCM_UNDEFINED));
 
   ruin_util_log(w, "dispatching event sdom:event-keyup for key '%s'\n",
-		SCM_STRING_CHARS(key_str));
+		scm_to_locale_string(key_str));
 
   scm_apply_0
     (scm_c_eval_string("sdom:dispatch-event"), 
@@ -390,6 +395,11 @@ void ruin_input_send(ruin_window_t *w, int ch) {
 		SCM_EOL, SCM_EOL, key_str, key_loc, SCM_BOOL_F, SCM_BOOL_F, 
 		SCM_BOOL_F, SCM_BOOL_F, SCM_UNDEFINED));
 
+  if (w->render_requested) {
+    ruin_load_layout_and_render(w);
+  }
+
+  w->render_requested = FALSE;
   return;
 }
 
