@@ -1,5 +1,5 @@
 /* libruin.h: Main exports for libRUIN
- * Copyright (C) 2005 Julian Graham
+ * Copyright (C) 2007 Julian Graham
  *
  * libRUIN is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -25,16 +25,11 @@
 
 typedef struct _ruin_window_t ruin_window_t;
 
-enum ruin_xml_dialect {
-  RUIN_XML_DIALECT_XUL,
-  RUIN_XML_DIALECT_XHTML
-};
-
 int ruin_init();
 
-int ruin_draw(ruin_window_t *, SCM, enum ruin_xml_dialect);
-int ruin_draw_string(ruin_window_t *, char *, enum ruin_xml_dialect);
-int ruin_draw_file(ruin_window_t *, char *, enum ruin_xml_dialect);
+int ruin_draw(ruin_window_t *, SCM);
+int ruin_draw_string(ruin_window_t *, char *);
+int ruin_draw_file(ruin_window_t *, char *);
 
 ruin_window_t *ruin_window_new(WINDOW *, FILE *);
 void ruin_window_free(ruin_window_t *);
