@@ -1,5 +1,5 @@
 /* render.c: Rendering and sizing routines for libRUIN
- * Copyright (C) 2006 Julian Graham
+ * Copyright (C) 2007 Julian Graham
  *
  * libRUIN is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -27,6 +27,7 @@
 #include "css.h"
 #include "layout.h"
 #include "render.h"
+#include "util.h"
 #include "window.h"
 
 void _ruin_render_set_colors(enum ruin_layout_foreground_color fg,
@@ -96,20 +97,20 @@ void _ruin_render_set_colors(enum ruin_layout_foreground_color fg,
   }
 }
 
-void _ruin_render_set_attrs(ruin_element_t *tree) {
+void _ruin_render_set_attrs(ruin_element_t *tree, ruin_util_list *inh) {
   /* If text-decoration is "none" on this element, look for inline ancestors
      that have a non-none text-decoration and use that... */
 
-  char *td = ruin_css_lookup(tree, "text-decoration");
+  char *td = ruin_css_lookup(tree, "text-decoration", inh);
   if (strcmp(td, "none") == 0) {
-    char *display = ruin_css_lookup(tree, "display");
+    char *display = ruin_css_lookup(tree, "display", inh);
     if (strcmp(display, "inline") != 0) {
-      display = ruin_css_lookup(tree->parent, "display");
+      display = ruin_css_lookup(tree->parent, "display", inh);
       if (strcmp(display, "inline") != 0)
 	return;
-      else _ruin_render_set_attrs(tree->parent);
+      else _ruin_render_set_attrs(tree->parent, inh);
     }
-    else _ruin_render_set_attrs(tree->parent);
+    else _ruin_render_set_attrs(tree->parent, inh);
   }
   else {
     if ((strstr(td, "underline") != NULL) ||
@@ -126,12 +127,13 @@ void _ruin_render_set_attrs(ruin_element_t *tree) {
    information; returns NULL if it crosses a non-inline element during the
    search. */
 
-ruin_element_t *_get_previous_anon_inline_sibling(ruin_element_t *tree) {
-  char *display = ruin_css_lookup(tree, "display");
+static ruin_element_t *_get_previous_anon_inline_sibling
+(ruin_element_t *tree, ruin_util_list *inh) {
+  char *display = ruin_css_lookup(tree, "display", inh);
   if (strcmp(display, "inline") != 0)
     return NULL;
   if (tree->prev_sibling != NULL) {
-    display = ruin_css_lookup(tree->prev_sibling, "display");
+    display = ruin_css_lookup(tree->prev_sibling, "display", inh);
     if (strcmp(display, "inline") != 0) {
       ruin_element_t *elt_ptr = tree->prev_sibling;
       while((elt_ptr != NULL) && 
@@ -145,7 +147,7 @@ ruin_element_t *_get_previous_anon_inline_sibling(ruin_element_t *tree) {
       return elt_ptr;
     }
   }
-  return _get_previous_anon_inline_sibling(tree->parent);
+  return _get_previous_anon_inline_sibling(tree->parent, inh);
 }
 
 /* Split the content into whitespace-delimited words; allow whitespace at the
@@ -241,41 +243,44 @@ void ruin_render_draw_inline(ruin_element_t *tree, ruin_util_list *inh) {
 
   char **lines = NULL;
   int num_lines;
-  int current_line_length;
-  int current_line_words;
+  int current_line_length = tree->first_line_start - tree->left;
   int current_line = 0;
 
   int ls = tree->letter_spacing.used, ws = tree->word_spacing.used;
+  int container_width = 
+    ruin_layout_find_containing_block(inh, mask)->width.used;
 
+  int prev_sib_ended_in_ws = FALSE;
+  
   if ((max_line_length <= 0) || (max_lines <= 0))
     return;
 
   if (tree->content != NULL &&
       (num_words = ruin_render_get_words
        (tree->content, &words, &word_lens)) > 0) {
-    char *wstype = ruin_css_lookup(tree, "white-space");
+    char *wstype = ruin_css_lookup(tree, "white-space", inh);
     
-    current_line_length = 0;
-    current_line_words = 0;
     num_lines = 1;
 
     lines = malloc(sizeof(char *) * tree->height.used);
     
     if (strcmp(wstype, "normal") == 0) {
-      char *buf = calloc(tree->width.used + 1, sizeof(char));
-      int buf_len = tree->width.used;
+      char *buf = calloc(container_width + 1, sizeof(char));
+      int buf_len = container_width;
       int num_words = ruin_render_get_words(tree->content, &words, &word_lens);
 
       lines[current_line] = buf;
 
-      if ((isspace(words[0][0])) && (current_line == 0)) {
+      if (isspace(words[0][0]) && 
+	  (!tree->prev_was_inline || (tree->prev_was_inline && 
+				      prev_sib_ended_in_ws))) {
 	word_lens[0]--;
 	words[0]++;
       }
 
       if ((tree->first_child == NULL) && (tree->next_sibling == NULL) &&
 	  (isspace(words[num_words - 1][word_lens[num_words - 1] - 1]))) {
-	words[num_words - 1][word_lens[num_words - 1] - 1] = 0;
+	/*	words[num_words - 1][word_lens[num_words - 1] - 1] = 0; */
 	word_lens[num_words - 1]--;
       }
 
@@ -285,13 +290,13 @@ void ruin_render_draw_inline(ruin_element_t *tree, ruin_util_list *inh) {
 	int addition = 0;
 	int ws_p = i == num_words - 1 ? 0 : ws;
 	
-	if (current_line_length + total_wlen > tree->width.used) {
+	if (current_line_length + total_wlen > container_width) {
 	  num_lines++;
 
 	  /* The current word won't fit on the current line... Will it fit on
 	     any line? */
 
-	  if (total_wlen > tree->width.used) {
+	  if (total_wlen > container_width) {
 
 	    /* We just cram it in... */
 
@@ -305,9 +310,9 @@ void ruin_render_draw_inline(ruin_element_t *tree, ruin_util_list *inh) {
 	      remainder -= addition;
 	      if (buf_len == 0) {
 		if (current_line < tree->height.used - 1) {
-		  buf = calloc(tree->width.used, sizeof(char));
+		  buf = calloc(container_width + 1, sizeof(char));
 		  lines[++current_line] = buf;
-		  buf_len = tree->width.used;
+		  buf_len = container_width;
 		  current_line_length = 0;
 		} else break;
 	      }
@@ -317,9 +322,9 @@ void ruin_render_draw_inline(ruin_element_t *tree, ruin_util_list *inh) {
 
 	    /* Start a new line and put the word there. */
 
-	    buf = calloc(tree->width.used, sizeof(char));
+	    buf = calloc(container_width + 1, sizeof(char));
 	    lines[++current_line] = buf;
-	    buf_len = tree->width.used;
+	    buf_len = container_width;
 	    addition = _ruin_render_add_word
 	      (buf, buf_len, words[i], wlen, ls, ws_p);
 	    buf += addition;
@@ -373,71 +378,92 @@ void ruin_render_draw_inline(ruin_element_t *tree, ruin_util_list *inh) {
        line. */
 
     container = ruin_layout_find_containing_block(inh, mask);
-    ruin_util_list_push_front
-      (container->current_pseudo_elements, "first-line");
-    ruin_util_list_push_front
-      (container->current_pseudo_elements, "first-letter");
+
+    tree->parent_window->current_pseudo_elements = 
+      ruin_util_list_push_front(tree->parent_window->current_pseudo_elements, 
+				ruin_util_list_new(strdup("first-line")));
+
+    tree->parent_window->current_pseudo_elements = 
+      ruin_util_list_push_front(tree->parent_window->current_pseudo_elements, 
+				ruin_util_list_new(strdup("first-letter")));
+
     ruin_css_clear_style_cache(container);
 
-    temp_fg = ruin_css_match_foreground_color(ruin_css_lookup(tree, "color"));
+    temp_fg = ruin_css_match_foreground_color
+      (ruin_css_lookup(tree, "color", inh));
     temp_bg = ruin_css_match_background_color
-      (ruin_css_lookup(tree, "background-color"), inh);
+      (ruin_css_lookup(tree, "background-color", inh), inh);
     _ruin_render_set_colors(temp_fg, temp_bg);
-    _ruin_render_set_attrs(tree);
+    _ruin_render_set_attrs(tree, inh);
 
     /* Draw the first letter */
+    if (tree->top > tree->parent_window->top->height.used)
+      return;
+
     move(tree->top, tree->first_line_start + tree->parent->text_indent.used);
     
-    if (strcmp(ruin_css_lookup(tree, "font-variant"), "small-caps") == 0)
+    if (strcmp(ruin_css_lookup(tree, "font-variant", inh), "small-caps") == 0)
       addch(toupper(lines[0][0]));
     else addch(lines[0][0]);
 
-    (void) ruin_util_list_pop_front(container->current_pseudo_elements);
+    /* Memory leak? */
+    tree->parent_window->current_pseudo_elements = 
+      tree->parent_window->current_pseudo_elements->next;
+
     ruin_css_clear_style_cache(container);
-    temp_fg = ruin_css_match_foreground_color(ruin_css_lookup(tree, "color"));
+    temp_fg = ruin_css_match_foreground_color
+      (ruin_css_lookup(tree, "color", inh));
     temp_bg = ruin_css_match_background_color
-      (ruin_css_lookup(tree, "background-color"), inh);
+      (ruin_css_lookup(tree, "background-color", inh), inh);
     _ruin_render_set_colors(temp_fg, temp_bg);
-    _ruin_render_set_attrs(tree);
+    _ruin_render_set_attrs(tree, inh);
 
     /* Draw the first line */
     move(tree->top, 
 	 tree->first_line_start + tree->parent->text_indent.used + 1);
-    if (strcmp(ruin_css_lookup(tree, "font-variant"), "small-caps") == 0) {
+    if (strcmp(ruin_css_lookup(tree, "font-variant", inh), 
+	       "small-caps") == 0) {
       int len = strlen(lines[0]);
       for (i = 1; i < len; i++)
 	lines[0][i] = toupper(lines[0][i]);
     }
     addstr(lines[0] + 1);
 
-    (void) ruin_util_list_pop_front(container->current_pseudo_elements);
+    /* Memory leak? */
+    tree->parent_window->current_pseudo_elements = 
+      tree->parent_window->current_pseudo_elements->next;
+
     ruin_css_clear_style_cache(container);
 
     _ruin_render_set_colors(tree->color, tree->background_color);
-    _ruin_render_set_attrs(tree);
+    _ruin_render_set_attrs(tree, inh);
     
     for (i = 1; i < num_lines; i++) {
       int new_top = 0, new_left = 0;
-      char *a = ruin_css_lookup(tree, "text-align");
+      char *a = ruin_css_lookup(tree, "text-align", inh);
       if (strcmp(a, "left") == 0) new_left = tree->left;
       else if (strcmp(a, "center") == 0) 
 	new_left = 
-	  tree->left + ((tree->width.used - (int) strlen(lines[i])) / 2); 
+	  tree->left + ((container_width - (int) strlen(lines[i])) / 2); 
       else if (strcmp(a, "right") == 0)
-	new_left = tree->left + tree->width.used - (int) strlen(lines[i]);
+	new_left = tree->left + container_width - (int) strlen(lines[i]);
       
       /* Need to handle the more complicated vertical alignments. */
       
-      a = ruin_css_lookup(tree, "vertical-align");
+      a = ruin_css_lookup(tree, "vertical-align", inh);
       if (strcmp(a, "top") == 0) new_top = tree->top + i;
       else if (strcmp(a, "middle") == 0)
 	new_top = tree->top + ((tree->height.used) / 2) - (i - num_lines / 2); 
       else if ((strcmp(a, "baseline") == 0) ||
 	       (strcmp(a, "bottom") == 0))
 	new_top = tree->top + tree->height.used - (num_lines - i);
+
+      if (new_top > tree->parent_window->top->height.used)
+	return;
       
       move(new_top, new_left);
-      if (strcmp(ruin_css_lookup(tree, "font-variant"), "small-caps") == 0) {
+      if (strcmp(ruin_css_lookup(tree, "font-variant", inh), 
+		 "small-caps") == 0) {
 	int len = strlen(lines[i]), j = 0;
 	for (j = 0; j < len; j++)
 	  lines[i][j] = toupper(lines[i][j]);
@@ -446,17 +472,16 @@ void ruin_render_draw_inline(ruin_element_t *tree, ruin_util_list *inh) {
       free(lines[i]);
     }
     free(lines);
-  }
-
-  { ruin_element_t *elt_ptr = tree->first_child;
-    ruin_util_list_push_front(inh, ruin_util_ptr_to_string(tree));
+  } else {
+    ruin_element_t *elt_ptr = tree->first_child;
+    inh = ruin_util_list_push_front(inh, ruin_util_list_new(tree));
     while(elt_ptr != NULL) {
-      if (strcmp(ruin_css_lookup(elt_ptr, "display"), "inline") == 0)
+      if (strcmp(ruin_css_lookup(elt_ptr, "display", inh), "inline") == 0)
 	ruin_render_render_tree(elt_ptr, inh);
       elt_ptr = elt_ptr->next_sibling;
     }
-    (void) ruin_util_list_pop_front(inh);
-  }
+    free(inh);
+  } 
 
   return;
 }
@@ -508,7 +533,8 @@ int _get_border_char(char *s, enum _ruin_render_border_char c) {
   } else return ' ';
 }
 
-void _ruin_render_draw_border(ruin_element_t *tree, int top, int left) {
+static void _ruin_render_draw_border
+(ruin_element_t *tree, ruin_util_list *inh, int top, int left) {
   int i = 0;
   int w = tree->width.used + tree->padding_left.used + 
     tree->padding_right.used + tree->border_left_width.used + 
@@ -517,7 +543,7 @@ void _ruin_render_draw_border(ruin_element_t *tree, int top, int left) {
     tree->padding_bottom.used + tree->border_top_width.used +
     tree->border_bottom_width.used;
 
-  char *border_style = ruin_css_lookup(tree, "border-top-style");
+  char *border_style = ruin_css_lookup(tree, "border-top-style", inh);
 
   /* Draw the top... */
 
@@ -546,7 +572,7 @@ void _ruin_render_draw_border(ruin_element_t *tree, int top, int left) {
 
   /* Draw the bottom... */
   
-  border_style = ruin_css_lookup(tree, "border-bottom-style");
+  border_style = ruin_css_lookup(tree, "border-bottom-style", inh);
   if ((strcmp(border_style, "none") != 0) &&
       (tree->border_bottom_width.used > 0)) {
 
@@ -576,7 +602,7 @@ void _ruin_render_draw_border(ruin_element_t *tree, int top, int left) {
 
   /* Draw the left... */
 
-  if ((strcmp(border_style = ruin_css_lookup(tree, "border-left-style"), 
+  if ((strcmp(border_style = ruin_css_lookup(tree, "border-left-style", inh), 
 	      "none") != 0) && (tree->border_left_width.used > 0)) {
 
     if (strcmp(border_style, "inset") == 0) {
@@ -597,7 +623,7 @@ void _ruin_render_draw_border(ruin_element_t *tree, int top, int left) {
 
   /* Draw the right... */
 
-  if ((strcmp(border_style = ruin_css_lookup(tree, "border-right-style"),
+  if ((strcmp(border_style = ruin_css_lookup(tree, "border-right-style", inh),
 	      "none") != 0) && (tree->border_right_width.used > 0)) {
 
     if (strcmp(border_style, "inset") == 0) {
@@ -658,6 +684,9 @@ void ruin_render_draw_block(ruin_element_t *tree, ruin_util_list *inh) {
     char *line = calloc(width + 1, sizeof(char));
     (void) memset(line, (int) ' ', width);
     for (i = 0; i < num_lines; i++) {
+      int top = render_top - tree->border_top_width.used + i;
+      if (top > tree->parent_window->top->height.used)
+	break;
       move(render_top - tree->border_top_width.used + i, 
 	   render_left - tree->border_left_width.used);
       addstr(line);
@@ -710,12 +739,13 @@ void ruin_render_draw_block(ruin_element_t *tree, ruin_util_list *inh) {
     ruin_render_render_tree(tree->caption, inh);
 
   { ruin_element_t *elt_ptr = tree->first_child;
-    ruin_util_list_push_front(inh, ruin_util_ptr_to_string(tree));
+    ruin_util_list *inh2 = ruin_util_list_new(tree);
+    inh2->next = inh;
     while(elt_ptr != NULL) {
-      ruin_render_render_tree(elt_ptr, inh);
+      ruin_render_render_tree(elt_ptr, inh2);
       elt_ptr = elt_ptr->next_sibling; 
     }
-    (void) ruin_util_list_pop_front(inh);
+    free(inh2);
   }
 
   /* The children might have changed the colors... */
@@ -724,36 +754,41 @@ void ruin_render_draw_block(ruin_element_t *tree, ruin_util_list *inh) {
   
   /* Draw the border. */
 
-  _ruin_render_draw_border(tree, render_top - tree->border_top_width.used, 
+  _ruin_render_draw_border(tree, inh, render_top - tree->border_top_width.used,
 			   render_left - tree->border_left_width.used);
-  
   return;
 }
 
 void ruin_render_draw_table(ruin_element_t *tree, ruin_util_list *inh) {
   ruin_element_t *elt_ptr = tree->first_child;
   while(elt_ptr != NULL) {
-    /* table row group */
-    ruin_element_t *elt_backup = elt_ptr;
-    elt_ptr = elt_ptr->first_child;
-    while(elt_ptr != NULL) {
-      /* table row */
-      ruin_element_t *elt_backup2 = elt_ptr;
-      ruin_util_list_push_front(inh, ruin_util_ptr_to_string(elt_ptr));
+    char *d = ruin_css_lookup(elt_ptr, "display", NULL);
+    if (strcmp(d, "table-row-group") == 0) {
+      ruin_element_t *elt_ptr_backup = elt_ptr;
+      elt_ptr = elt_ptr->first_child;
+      while(elt_ptr != NULL) {
+	ruin_element_t *elt_ptr_backup_backup = elt_ptr;
+	elt_ptr = elt_ptr->first_child;
+	while(elt_ptr != NULL) {
+	  ruin_render_render_tree(elt_ptr, inh);
+	  elt_ptr = elt_ptr->next_sibling;
+	}
+	elt_ptr = elt_ptr_backup_backup;
+	elt_ptr = elt_ptr->next_sibling;
+      }
+      elt_ptr = elt_ptr_backup;
+    } else if (strcmp(d, "table-row") == 0) {
+      ruin_element_t *elt_ptr_backup = elt_ptr;
       elt_ptr = elt_ptr->first_child;
       while(elt_ptr != NULL) {
-	/* table cell */
 	ruin_render_render_tree(elt_ptr, inh);
 	elt_ptr = elt_ptr->next_sibling;
       }
-      (void) ruin_util_list_pop_front(inh);
-      elt_ptr = elt_backup2;
-      elt_ptr = elt_ptr->next_sibling;
+      elt_ptr = elt_ptr_backup;
     }
-    elt_ptr = elt_backup;
     elt_ptr = elt_ptr->next_sibling;
   }
-  _ruin_render_draw_border(tree, tree->top, tree->left);
+  _ruin_render_draw_border(tree, inh, tree->top, tree->left);
   return;
 }
 
@@ -761,7 +796,7 @@ void ruin_render_draw_list_item(ruin_element_t *tree, ruin_util_list *inh) {
   int render_top, render_left;
   int my_pos = 1, has_dot = TRUE;
   char *marker = NULL;
-  char *style = ruin_css_lookup(tree, "list-style-type");
+  char *style = ruin_css_lookup(tree, "list-style-type", inh);
 
   ruin_element_t *elt_ptr = tree->prev_sibling;
   while(elt_ptr != NULL) {
@@ -833,31 +868,32 @@ void ruin_render_draw_list_item(ruin_element_t *tree, ruin_util_list *inh) {
   /* Draw the children... */
 
   elt_ptr = tree->first_child;
-  ruin_util_list_push_front(inh, ruin_util_ptr_to_string(tree));
+
+  inh = ruin_util_list_push_front(inh, ruin_util_list_new(tree));
   while(elt_ptr != NULL) {
     ruin_render_render_tree(elt_ptr, inh);
     elt_ptr = elt_ptr->next_sibling;
   }
-  (void) ruin_util_list_pop_front(inh);
+  free(inh);
 
   return;
 }
 
 void ruin_render_render_tree(ruin_element_t *tree, ruin_util_list *inh) {
-  char *d = ruin_css_lookup(tree, "display");
+  char *d = ruin_css_lookup(tree, "display", inh);
 
   tree->color = ruin_css_match_foreground_color
-    (ruin_css_lookup(tree, "color"));
+    (ruin_css_lookup(tree, "color", inh));
   tree->background_color = ruin_css_match_background_color
-    (ruin_css_lookup(tree, "background-color"), inh);
+    (ruin_css_lookup(tree, "background-color", inh), inh);
   tree->border_top_color = ruin_css_match_foreground_color
-    (ruin_css_lookup(tree, "border-top-color"));
+    (ruin_css_lookup(tree, "border-top-color", inh));
   tree->border_left_color = ruin_css_match_foreground_color
-    (ruin_css_lookup(tree, "border-left-color"));
+    (ruin_css_lookup(tree, "border-left-color", inh));
   tree->border_bottom_color = ruin_css_match_foreground_color
-    (ruin_css_lookup(tree, "border-bottom-color"));
+    (ruin_css_lookup(tree, "border-bottom-color", inh));
   tree->border_right_color = ruin_css_match_foreground_color
-    (ruin_css_lookup(tree, "border-right-color"));
+    (ruin_css_lookup(tree, "border-right-color", inh));
 
   if ((strcmp(d, "block") == 0) ||
       (strcmp(d, "table-cell") == 0)) {
