@@ -1,5 +1,5 @@
 /* xhtml.c: XHTML-parsing support for libRUIN
- * Copyright (C) 2006 Julian Graham
+ * Copyright (C) 2007 Julian Graham
  *
  * libRUIN is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -33,7 +33,7 @@
 void ruin_xhtml_generate_tree_parse_attrs(ruin_element_t *t) {
   SCM attr_str = scm_call_2(scm_c_eval_string("sdom:get-attribute"),
 			    t->element, scm_makfrom0str("style"));
-  if (scm_eq_p(attr_str, SCM_EOL) != SCM_BOOL_T) {
+  if (attr_str != SCM_BOOL_F) {
     SCM parsed_style = ruin_dialect_parse_in_document_style
       (t->doc, scm_string_append(scm_list_3(scm_makfrom0str("* {"),
 					    attr_str,
@@ -55,7 +55,7 @@ ruin_element_t *ruin_xhtml_generate_tree(ruin_window_t *w, SCM node,
   SCM node_next = SCM_EOL; 
   SCM node_type = SCM_EOL;
 
-  if (node == SCM_EOL)
+  if (node == SCM_BOOL_F)
     return NULL;
 
   node_next = scm_call_2(scm_c_eval_string("sdom:get-dom-property"), node,
@@ -162,7 +162,13 @@ ruin_element_t *ruin_xhtml_generate_tree(ruin_window_t *w, SCM node,
     if (my_result->focusable) {
       ruin_dialect_update_tab_position(my_result, 0);
     }
-    
+
+    if (strcmp(ruin_css_lookup(my_result, "display", NULL), "inline") == 0 &&
+	((sibling != NULL && strcmp(ruin_css_lookup(sibling, "display", NULL), 
+				   "inline") == 0) ||
+	 (parent != NULL && parent->prev_was_inline)))
+      my_result->prev_was_inline = TRUE;
+
     my_result->attributes = 
       scm_call_2(scm_c_eval_string("sdom:get-dom-property"),
 		 node, scm_makfrom0str("sdom:attributes"));
@@ -181,40 +187,22 @@ ruin_element_t *ruin_xhtml_generate_tree(ruin_window_t *w, SCM node,
 	my_result->first_child = children_result;
       }
     }
-
-    /* For some elements, we may need to do some post-processing... TK */
-
-    /* scm_remember_upto_here_1(local_name_scm);
-       scm_remember_upto_here_1(node_name_scm); */
   }    
 
+  ruin_dialect_add_table_parents(my_result);
+  ruin_dialect_add_table_children(my_result);
+  ruin_dialect_add_table_columns(my_result);
+
   if ((my_result != NULL) && (!not_sibling)) {
     ruin_element_t *return_ptr = my_result;
     ruin_element_t *parent_ptr = my_result->parent;
     ruin_util_hash_insert
       (w->internal_ids, ruin_util_long_to_string(my_result->internal_id),
-       ruin_util_ptr_to_string((void *) my_result));
+       my_result);
 
-    /* Now we need to add intervening table elements... */
-    return_ptr = ruin_dialect_add_table_nodes(my_result);
-
-    if ((parent_ptr != my_result->parent) && 
-	(strcmp(ruin_css_lookup(my_result->parent, "display"),
-		"table-cell") == 0)) {
-
-      /* If we've inserted a cell, we don't want to pass the cell as the
-	 parent pointer because then all the siblings will go into the same
-	 cell.  Also, we need to set the sibling pointer to be the newly-
-         inserted cell element. */
-      
-      parent_ptr = my_result->parent->parent;
-      my_result->parent->next_sibling = 
-	ruin_xhtml_generate_tree(w, node_next, parent_ptr, my_result->parent);
-    }
-    else {
-      my_result->next_sibling = 
+    my_result->next_sibling = 
 	ruin_xhtml_generate_tree(w, node_next, parent_ptr, my_result);
-    }
+
     return return_ptr;
   } else return ruin_xhtml_generate_tree(w, node_next, parent, sibling);
 }
