@@ -1,5 +1,5 @@
 /* xul.c: XUL-parsing support for libRUIN
- * Copyright (C) 2006 Julian Graham
+ * Copyright (C) 2007 Julian Graham
  *
  * libRUIN is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -43,7 +43,7 @@ void _ruin_generate_tree_parse_attrs(SCM doc, SCM node, SCM parent,
   SCM ns_name_scm = SCM_EOL;
   SCM attr_str_scm = SCM_EOL;
 
-  if ((node == SCM_EOL) || (t == NULL))
+  if ((node == SCM_BOOL_F) || (t == NULL))
     return;
 
   /* TODO: This is a hack -- we're just clobbering the parent's namespace at
@@ -51,7 +51,7 @@ void _ruin_generate_tree_parse_attrs(SCM doc, SCM node, SCM parent,
 
   plocal_name_scm = scm_call_2(scm_c_eval_string("sdom:get-dom-property"),
 			       parent, scm_makfrom0str("sdom:local-name"));
-  if (plocal_name_scm == SCM_EOL) {
+  if (plocal_name_scm == SCM_BOOL_F) {
     plocal_name_scm =  
       scm_call_2(scm_c_eval_string("sdom:get-dom-property"), parent, 
 		 scm_makfrom0str("sdom:tag-name"));
@@ -63,7 +63,7 @@ void _ruin_generate_tree_parse_attrs(SCM doc, SCM node, SCM parent,
 
   local_name_scm = scm_call_2(scm_c_eval_string("sdom:get-dom-property"),
 			      node, scm_makfrom0str("sdom:local-name"));
-  if (local_name_scm == SCM_EOL) {
+  if (local_name_scm == SCM_BOOL_F) {
     local_name_scm = scm_call_2
       (scm_c_eval_string("sdom:get-dom-property"), node, 
        scm_makfrom0str("sdom:name"));
@@ -195,23 +195,13 @@ void _ruin_generate_tree_parse_attrs(SCM doc, SCM node, SCM parent,
   } else if (strcmp((char *) attr_name, "flags") == 0) {
   } else if (strcmp((char *) attr_name, "flex") == 0) {
   } else if (strcmp((char *) attr_name, "height") == 0) {
-    /*    int height = 0;
-    int attr_str_len = strlen((char *) attr_str);
-    if (sscanf((char *) attr_str, "%d", &height) == 1) {
-      t->height.computed = height;
-      t->height.units = RUIN_LAYOUT_UNITS_CHARS;
-      if ((attr_str_len > 1) && 
-	  (strcmp(attr_str + attr_str_len - 1, "%") == 0))
-	t->height.units = RUIN_LAYOUT_UNITS_PERCENT;
-	} */
     ruin_layout_add_style(&t->additional_attribute_style, "height", attr_str);
   } else if (strcmp((char *) attr_name, "hidden") == 0) {
   } else if (strcmp((char *) attr_name, "id") == 0) {
     if (strlen(attr_str) > 0) {
       free(t->id);
       t->id = strdup(attr_str);
-      ruin_util_hash_insert
-	(t->ids, attr_str, ruin_util_ptr_to_string((void *) t));
+      ruin_util_hash_insert(t->ids, attr_str, t);
     }
   } else if (strcmp((char *) attr_name, "insertafter") == 0) {
   } else if (strcmp((char *) attr_name, "inserbefore") == 0) {
@@ -268,15 +258,6 @@ void _ruin_generate_tree_parse_attrs(SCM doc, SCM node, SCM parent,
   } else if (strcmp((char *) attr_name, "uri") == 0) {
   } else if (strcmp((char *) attr_name, "wait-cursor") == 0) {
   } else if (strcmp((char *) attr_name, "width") == 0) {
-    /* int width = 0;
-    int attr_str_len = strlen((char *) attr_str);
-    if (sscanf((char *) attr_str, "%d", &width) == 1) {
-      t->width.computed = width;
-      t->width.units = RUIN_LAYOUT_UNITS_CHARS;
-      if ((attr_str_len > 1) && 
-	  (strcmp(attr_str + attr_str_len - 1, "%") == 0))
-	t->width.units = RUIN_LAYOUT_UNITS_PERCENT;
-    } */
     ruin_layout_add_style(&t->additional_attribute_style, "width", attr_str);
   }
 
@@ -370,7 +351,7 @@ ruin_element_t *ruin_xul_generate_tree(ruin_window_t *w, SCM node,
   SCM node_next = SCM_EOL; 
   SCM node_type = SCM_EOL;
 
-  if (node == SCM_EOL)
+  if (node == SCM_BOOL_F)
     return NULL;
 
   node_next = scm_call_2(scm_c_eval_string("sdom:get-dom-property"), node,
@@ -428,22 +409,13 @@ ruin_element_t *ruin_xul_generate_tree(ruin_window_t *w, SCM node,
     else if (strcmp((char *) node_name, "checkbox") == 0) {
       my_result->focusable = TRUE;
       my_result->extra_content = RUIN_LAYOUT_EXTRA_CONTENT_CHECKBOX;
-    }
-    /* else if (strcmp((char *) node_name, "caption") == 0) {
-      if ((parent != NULL) && (parent->caption == NULL)) {
-	parent->caption = my_result;	
-	not_sibling = TRUE;
-      }
-      } */
-    else if (strcmp((char *) node_name, "colorpicker") == 0) {
+    } else if (strcmp((char *) node_name, "colorpicker") == 0) {
       my_result->focusable = TRUE;
       my_result->extra_content = RUIN_LAYOUT_EXTRA_CONTENT_COLORPICKER;
-    }
-    else if (strcmp((char *) node_name, "radio") == 0) {
+    } else if (strcmp((char *) node_name, "radio") == 0) {
       my_result->focusable = TRUE;
       my_result->extra_content = RUIN_LAYOUT_EXTRA_CONTENT_RADIO;
-    }
-    else if (strcmp((char *) node_name, "textbox") == 0) {
+    } else if (strcmp((char *) node_name, "textbox") == 0) {
       my_result->focusable = TRUE;
       my_result->editable = RUIN_LAYOUT_EDITABLE_TYPE_SINGLE_LINE;
     }
@@ -466,7 +438,7 @@ ruin_element_t *ruin_xul_generate_tree(ruin_window_t *w, SCM node,
     
     { SCM children = scm_call_2(scm_c_eval_string("sdom:get-dom-property"),
 				node, scm_makfrom0str("sdom:child-nodes"));
-      if (children != SCM_EOL) {
+      if (children != SCM_BOOL_F) {
         children_result = ruin_xul_generate_tree
 	  (w, SCM_CAR(children), my_result, NULL); 
 	my_result->first_child = children_result;
@@ -508,10 +480,11 @@ ruin_element_t *ruin_xul_generate_tree(ruin_window_t *w, SCM node,
 	if (my_result->min_height.computed == RUIN_LAYOUT_VALUE_AUTO)
 	  my_result->min_height.computed = 3;
     }
-
-    /* scm_remember_upto_here_1(local_name_scm);
-       scm_remember_upto_here_1(node_name_scm); */
   }
+
+  ruin_dialect_add_table_parents(my_result);
+  ruin_dialect_add_table_children(my_result);
+  ruin_dialect_add_table_columns(my_result);
   
   if ((my_result != NULL) && (!not_sibling)) {
     ruin_element_t *return_ptr = my_result;
@@ -520,11 +493,8 @@ ruin_element_t *ruin_xul_generate_tree(ruin_window_t *w, SCM node,
       (w->internal_ids, ruin_util_long_to_string(my_result->internal_id),
        ruin_util_ptr_to_string((void *) my_result));
 
-    /* Now we need to add intervening table elements... */
-    return_ptr = ruin_dialect_add_table_nodes(my_result);
-    
     if ((parent_ptr != my_result->parent) && 
-	(strcmp(ruin_css_lookup(my_result->parent, "display"),
+	(strcmp(ruin_css_lookup(my_result->parent, "display", NULL),
 		"table-cell") == 0)) {
 
       /* If we've inserted a cell, we don't want to pass the cell as the
